<?php
/**********************************************************************
 * File  : dolibarr2ldap.php
 * Author: Eric SEIGNE
 *         mailto:eric.seigne@ryxeo.com
 *         http://www.ryxeo.com
 * Date  : 01/10/2007
 *
 * Description: 
 *      Copie tous les contacts de dolibarr dans une partie d'un
 *      annuaire LDAP, très pratique pour partager le fichier client
 *      avec les logiciels de mail par exemple
 *      Comme cette partie de l'annuaire est remise à zéro à chaque
 *      exécution de ce script il ne faut pas essayer de mettre à jour
 *      les données de l'annuaire, il faut juste faire les modifications
 *      dans dolibarr
 * 
 *********************************************************************/


//  ***********************************************************************  \\
// *      Copyright (c) 2007 by Eric Seigne <eric.seigne@ryxeo.com         * //
// *                                                                       * \\
// *    This program is free software; you can redistribute it and/or      * //
// *    modify it under the terms of the GNU General Public License as     * \\
// *    published by the Free Software Foundation; either version 2 of     * //
// *    the License, or (at your option) any later version.                * \\
// *                                                                       * //
// *    This program is distributed in the hope that it will be useful,    * \\
// *    but WITHOUT ANY WARRANTY; without even the implied warranty of     * //
// *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      * \\
// *    GNU General Public License for more details.                       * //
// *                                                                       * \\
// *  You should have received a copy of the GNU General Public License    * //
// *  along with this program in the file 'COPYING'; if not, write to      * \\
// *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,     * //
// *  Boston, MA 02111-1307, USA, or contact the author:                   * \\
// *                                                                       * //
// *                               Eric Seigne <eric.seigne@ryxeo.com>     * \\
// *                               http://www.ryxeo.com                    * //
// *                                                                       * \\
// ************************************************************************* //

// ************************* DÉBUT DE LA CONFIGURATION

// ----------------------
// CONFIGURATION LDAP
// ----------------------
//Adresse du serveur LDAP
$_ldap_server="ldap://localhost";
//Compte a utiliser pour la connexion, doit avoir les droits en écriture
$_ldap_connect="cn=admin,dc=nodomain";
//Le mot de passe du compte en question
$_ldap_password="ryxeopass";
//La base de l'annuaire LDAP
$_ldap_base="dc=nodomain";
//La branche de l'arbre ldap dans lequel se trouvera tous les contacts
//issus de dolibarr
$_ldap_dolibarr="dolibarr";
$_ldap_base_dolibarr="ou=$_ldap_dolibarr,$_ldap_base";

// ----------------------
// CONFIGURATION MYSQL
// ----------------------
$_mysql_server="localhost";            // Serveur de base de données
$_mysql_db="dolibarrstable";           // Nom de la base de données de dolibarr
$_mysql_user="dolibarruser";           // Login d'accès à cette base
$_mysql_password="ryxeopass";          // Mot de passe correspondant

// ************************* FIN DE LA CONFIGURATION

include "mysql.erics.php";


// ******************** DÉBUT DES FONCTIONS ******************* //
// utilisée pour supprimer recursivement tout le contenu de la
// branche dolibarr de l'annuaire ...
function myldap_delete($ds,$dn,$recursive=false){
  if($recursive == false){
    return(ldap_delete($ds,$dn));
  }else{
    //searching for sub entries
    $sr=ldap_list($ds,$dn,"ObjectClass=*",array(""));
    $info = ldap_get_entries($ds, $sr);
    for($i=0;$i<$info['count'];$i++){
      //deleting recursively sub entries
      $result=myldap_delete($ds,$info[$i]['dn'],$recursive);
      if(!$result){
	//return result code, if delete fails
	return($result);
      }
    }
    return(ldap_delete($ds,$dn));
  }
}

//fonction rapide pour faire un trim sur toutes les valeurs d'un tableau
function trim_value(&$value)
{
  $value = trim($value);
}
// ******************** FIN DES FONCTIONS ******************* //

$conn = SQL_Connect($_mysql_db,$_mysql_user,$_mysql_password,$_mysql_server);
$ds   = ldap_connect($_ldap_server)
  or die("Could not connect to $_ldap_server");

//SUPER IMPORTANT sinon ca ne bind pas !
ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);

if ($ds && $conn) {
    // bind with appropriate dn to give update access
    $r = ldap_bind($ds, $_ldap_connect, $_ldap_password);
    if($r)
      echo "LDAP Bind OK\n";
    else {
      echo "LDAP Bind ERR [$_ldap_server]:\n";
      switch (ldap_errno($r)) {
      case 0x31:
	echo "Bad username or password. Please try again.\n";
	break;
      case 0x32:
	echo "Insufficient access rights.\n";
	break;
      case -1:
	echo "Could not connect to on port \n";
	break;
      default:
	echo "Could not bind to the LDAP server.\n";
      }
      echo "ERR: " . ldap_error($ds) . " | " . ldap_errno($ds) . " | " . ldap_err2str($r) . "\n";
      break;
    }

    //On vide l'annuaire
    myldap_delete($ds,$_ldap_base_dolibarr,1);

    $info["ou"] = $_ldap_dolibarr;
    $info["objectclass"] = "organizationalUnit";
    $r = ldap_add($ds, $_ldap_base_dolibarr, $info);

    //Requete dans dolibarr pour les contacts
    $req = "select * from llx_socpeople,llx_societe WHERE fk_soc=llx_societe.idp";
    $res = sql_exec($conn,$req);

    $nbr = sql_numrows($res);
    $i = 0;
    while($i < $nbr) {
      $tab=sql_fetchrow($res,$i,1);
      unset($info);
      array_walk($tab, 'trim_value');
      if($tab["name"] != "") {
	// prepare data
	$info["cn"] = utf8_encode($tab["name"] . " " . $tab["firstname"]);
	$info["sn"] = utf8_encode($tab["name"]);
	if($tab["email"])
	  $info["mail"] = utf8_encode($tab["email"]);
	else
	  $info["mail"] = utf8_encode("none@none.com");
	if($tab["address"])
	  $info["street"] = utf8_encode($tab["address"]);
	if($tab["cp"])
	  $info["postalCode"] = utf8_encode($tab["cp"]);
	if($tab["nom"])
	  $info["o"] = utf8_encode($tab["nom"]);
	if($tab["ville"])
	  $info["l"] = utf8_encode($tab["ville"]);
	if($tab["phone"])
	  $info["telephoneNumber"] = utf8_encode($tab["phone"]);
	if($tab["phone_mobile"])
	  $info["mobile"] = utf8_encode($tab["phone_mobile"]);
	if($tab["fax"])
	  $info["facsimileTelephoneNumber"] = utf8_encode($tab["fax"]);
	if($tab["note"])
	  $info["description"] = utf8_encode($tab["note"]);
	//	$info[""] = $tab[""];
	$info["objectclass"] = "inetorgperson";
	// add data to directory
	if($tab["email"] != "")
	  $lemail = "+mail=" . $tab["email"];
	$r = ldap_add($ds, utf8_encode("cn=" . $tab["name"] . " " . $tab["firstname"] . $lemail . "," . $_ldap_base_dolibarr), $info);
	if($r == TRUE) {
	  $insert_ok++;
	  //      pour le debug ...
	  //	  print_r($info);
	}
	else {
	  $insert_error++;
	  //      pour le debug ...
	  echo "  " . ldap_error($ds) . " | " . ldap_errno($ds) . " | " . ldap_err2str($r) . "\n";
	  //print_r($info);
	}
      }
      $i++;
    }
    print "OK: $insert_ok\n";
    print "ERR: $insert_error -- pour les details regarder le code source [DEBUG] --\n";
} else {
    echo "Unable to connect to LDAP server or MYSQL server !";
}

/*
  echo "\n SEARCH \n";
  $filter="(|(cn=adm*))";
  $justthese = array("cn", "sn", "description", "mail");
  
  $sr = ldap_search($ds, "dc=nodomain", $filter, $justthese);
  $info = ldap_get_entries($ds, $sr);
  
  echo $info["count"]." entries returned\n";
  print_r($info);

  echo "\n";
*/

ldap_close($ds);
SQL_Close($conn);

?>
